from libc.stdio cimport FILE
from posix.types cimport off_t

from CoordinateList cimport CoordinateList

cdef extern from "gat_utils.h":
    long searchsorted(void * base,
                       size_t nmemb,
                       size_t size,
                       void * target,
                       int(*compar)(void *, void *))

    long searchargsorted(void * base,
                         int * sorted,
                         size_t nmemb,
                         size_t size,
                         void * target,
                         int(*compar)(void *, void *))

    int toCompressedFile(unsigned char *, size_t, FILE *)
    int fromCompressedFile(unsigned char *, size_t, FILE *)

cdef bytes force_bytes(object s, encoding=*)
cdef force_str(object s, encoding=*)

#####################################################
#####################################################
## type definitions
#####################################################
# type for positions
ctypedef unsigned int Position
# type for position difference - can be negative
ctypedef int PositionDifference

# A segment (without strand)
cdef struct Segment:
    Position start
    Position end

#####################################################
#####################################################
## definition of segmentlist
#####################################################
cdef class SegmentList(CoordinateList):

    cdef Segment * segments
    cdef size_t nsegments
    cdef size_t allocated
    cdef int flag

    # C and Python methods
    cpdef share(self, key)
    cpdef sort(self)
    cpdef SegmentList extend(self, SegmentList other)
    cpdef add(self, Position start, Position end)
    cpdef trim_ends(self, Position pos, Position size, int forward )
    cpdef trim(self, Position pos, Position size)
    cpdef normalize(self)
    cpdef merge(self, PositionDifference distance )
    cpdef check(self)
    cpdef Position getRandomPosition(self)
    cpdef Position overlapWithRange(
        self, Position start, Position end)
    cpdef SegmentList getOverlappingSegmentsWithRange(
        self, Position start, Position end)
    cpdef Position overlapWithSegments(
        self, SegmentList other)
    cpdef Position intersectionWithSegments(
        self, SegmentList other, mode = *)
    cpdef SegmentList getFilledSegmentsFromStart(
        self, Position start, PositionDifference remainder)
    cpdef SegmentList getFilledSegmentsFromEnd(
        self, Position end, PositionDifference remainder)
    cpdef void filter(self, SegmentList other)
    cpdef void intersect(self, SegmentList other)
    cpdef void subtract(self, SegmentList other)
    cpdef extend_segments(self, int extension)
    cpdef expand_segments(self, double expansion)
    cpdef Position sum(self)
    cpdef Position max(self)
    cpdef Position min(self)
    cpdef clear(self)
    cpdef summarize(self)
    cpdef SegmentList clone(self)

    # C only methods
    cdef _add(self, Segment segment)
    cdef _resize(self, int nsegments)
    cdef insert(self, int idx, Segment seg)
    cdef int _getInsertionPoint(self, Segment other)
    cdef Position overlap(self, Segment other)
    cdef SegmentList getOverlappingSegments(self, Segment other)
    cdef void truncate(self, Segment other)

    # Functions overloaded from CoordinateList
    cdef off_t toMMAP(self, void *, int, off_t)
    cdef void fromMMAP(self)




'''
Stats.py
======

statistical functions
'''

import math
import numpy
import types
from functools import reduce

try:
    import scipy.interpolate
    HAS_SCIPY = True
except ImportError:
    HAS_SCIPY = False


class FDRResult:

    def __init__(self):
        pass


def computeQValues(pvalues,
                   vlambda=None,
                   pi0_method="smoother",
                   fdr_level=None,
                   robust=False,
                   smooth_df=3,
                   smooth_log_pi0=False,
                   pi0=None):
    """compute qvalues after the method by Storey et al. (2002)

    The python code derives from the R implementation at 
    http://genomics.princeton.edu/storeylab/qvalue/linux.html.
    """

    if min(pvalues) < 0 or max(pvalues) > 1:
        raise ValueError("p-values out of range")

    m = len(pvalues)
    pvalues = numpy.array(pvalues, dtype=numpy.float)

    if vlambda == None:
        vlambda = numpy.arange(0, 0.95, 0.05)

    if pi0 == None:
        if type(vlambda) == float:
            vlambda = (vlambda,)

        if len(vlambda) > 1 and len(vlambda) < 4:
            raise ValueError(
                " if length of vlambda greater than 1, you need at least 4 values.")

        if len(vlambda) > 1 and (min(vlambda) < 0 or max(vlambda) >= 1):
            raise ValueError("vlambda must be within [0, 1).")

        # estimate pi0
        if len(vlambda) == 1:
            vlambda = vlambda[0]
            if vlambda < 0 or vlambda >= 1:
                raise ValueError("vlambda must be within [0, 1).")

            pi0 = numpy.mean([x >= vlambda for x in pvalues]) / (1.0 - vlambda)
            pi0 = min(pi0, 1.0)
        else:

            pi0 = numpy.zeros(len(vlambda), numpy.float)

            for i in range(len(vlambda)):
                pi0[i] = numpy.mean([x >= vlambda[i]
                                     for x in pvalues]) / (1.0 - vlambda[i])

            if pi0_method == "smoother":
                if smooth_log_pi0:
                    pi0 = numpy.log(pi0)
                if HAS_SCIPY:
                    tck = scipy.interpolate.splrep(
                        vlambda, pi0, k=smooth_df, s=10000)
                    pi0 = scipy.interpolate.splev(max(vlambda), tck)
                else:
                    raise ImportError("pi0_method smoother requires scipy")

                if smooth_log_pi0:
                    pi0 = numpy.exp(pi0)

            elif pi0_method == "bootstrap":
                minpi0 = min(pi0)

                mse = numpy.zeros(len(vlambda), numpy.float)
                pi0_boot = numpy.zeros(len(vlambda), numpy.float)

                for i in range(100):
                    # sample pvalues
                    idx_boot = numpy.random.random_integers(0, m - 1, m)
                    pvalues_boot = pvalues[idx_boot]

                    for x in range(len(vlambda)):
                        # compute number of pvalues larger than lambda[x]
                        pi0_boot[x] = numpy.mean(
                            pvalues_boot > vlambda[x]) / (1.0 - vlambda[x])
                    mse += (pi0_boot - minpi0) ** 2
                pi0 = min(pi0[mse == min(mse)])
            else:
                raise ValueError(
                    "'pi0_method' must be one of 'smoother' or 'bootstrap'.")

            pi0 = min(pi0, 1.0)

    if pi0 <= 0:
        raise ValueError(
            "The estimated pi0 <= 0 (%f). Check that you have valid p-values or use another vlambda method." % pi0)

    if fdr_level != None and (fdr_level <= 0 or fdr_level > 1):
        raise ValueError("'fdr_level' must be within (0, 1].")

    # compute qvalues

    idx = numpy.argsort(pvalues)
    # monotonically decreasing bins, so that bins[i-1] > x >=  bins[i]
    bins = numpy.unique(pvalues)[::-1]

    # v[i] = number of observations less than or equal to pvalue[i]
    # could this be done more elegantly?
    val2bin = len(bins) - numpy.digitize(pvalues, bins)
    v = numpy.zeros(m, dtype=numpy.int)
    lastbin = None
    for x in range(m - 1, -1, -1):
        bin = val2bin[idx[x]]
        if bin != lastbin:
            c = x
        v[idx[x]] = c + 1
        lastbin = bin

    qvalues = pvalues * pi0 * m / v
    if robust:
        qvalues /= (1.0 - (1.0 - pvalues) ** m)

    # bound qvalues by 1 and make them monotonic
    qvalues[idx[m - 1]] = min(qvalues[idx[m - 1]], 1.0)
    for i in range(m - 2, -1, -1):
        qvalues[idx[i]] = min(min(qvalues[idx[i]], qvalues[idx[i + 1]]), 1.0)

    # fill result
    result = FDRResult()
    result.qvalues = qvalues

    if fdr_level != None:
        result.passed = [x <= fdr_level for x in result.qvalues]
    else:
        result.passed = [False for x in result.qvalues]

    result.pvalues = pvalues
    result.pi0 = pi0
    result.vlambda = vlambda
    result.fdr_level = fdr_level

    return result


# {{{ http://code.activestate.com/recipes/511478/ (r1)
def percentile(N, percent, key=lambda x: x):
    """
    Find the percentile of a list of values.

    @parameter N - is a list of values. Note N MUST BE already sorted.
    @parameter percent - a float value from 0.0 to 1.0.
    @parameter key - optional key function to compute value from each element of N.

    @return - the percentile of the values
    """
    if len(N) == 0:
        return None
    k = (len(N) - 1) * percent
    f = math.floor(k)
    c = math.ceil(k)
    if f == c:
        return key(N[int(k)])
    d0 = key(N[int(f)]) * (c - k)
    d1 = key(N[int(c)]) * (k - f)
    return d0 + d1

###################################################################
###################################################################
###################################################################
# adjust P-Value
###################################################################


def adjustPValues(pvalues, method='fdr', n=None):
    '''returns an array of adjusted pvalues

    Reimplementation of p.adjust in the R package.

    p: numeric vector of p-values (possibly with 'NA's).  Any other
    R is coerced by 'as.numeric'.

    method: correction method. Valid values are:

    n: number of comparisons, must be at least 'length(p)'; only set
    this (to non-default) when you know what you are doing

    For more information, see the documentation of the
    p.adjust method in R.
    '''

    if n == None:
        n = len(pvalues)

    if method == "fdr":
        method = "BH"

    # optional, remove NA values
    p = numpy.array(pvalues, dtype=numpy.float)
    lp = len(p)

    assert n <= lp

    if n <= 1:
        return p
    if n == 2 and method == "hommel":
        method = "hochberg"

    if method == "bonferroni":
        p0 = n * p
    elif method == "holm":
        i = numpy.arange(lp)
        o = numpy.argsort(p)
        ro = numpy.argsort(o)
        m = numpy.maximum.accumulate((n - i) * p[o])
        p0 = m[ro]
    elif method == "hommel":
        raise NotImplementedError("hommel method not fully implemented")
    elif method == "hochberg":
        i = numpy.arange(0, lp)[::-1]
        o = numpy.argsort(1 - p)
        ro = numpy.argsort(o)
        m = numpy.minimum.accumulate((n - i) * p[o])
        p0 = m[ro]
    elif method == "BH":
        i = numpy.arange(1, lp + 1)[::-1]
        o = numpy.argsort(1 - p)
        ro = numpy.argsort(o)
        m = numpy.minimum.accumulate(float(n) / i * p[o])
        p0 = m[ro]
    elif method == "BY":
        i = numpy.arange(1, lp + 1)[::-1]
        o = numpy.argsort(1 - p)
        ro = numpy.argsort(o)
        q = numpy.sum(1.0 / numpy.arange(1, n + 1))
        m = numpy.minimum.accumulate(q * float(n) / i * p[o])
        p0 = m[ro]
    elif method == "none":
        p0 = p

    return numpy.minimum(p0, numpy.ones(len(p0)))


class Result(object):

    '''allow both member and dictionary access.'''
    slots = ("_data")

    def __init__(self):
        object.__setattr__(self, "_data", dict())

    def fromR(self, take, r_result):
        '''convert from an *r_result* dictionary using map *take*.

        *take* is a list of tuples mapping a field to the corresponding
        field in *r_result*.
        '''
        for x, y in take:
            if y:
                self._data[x] = r_result.rx(y)[0][0]
            else:
                self._data[x] = r_result.rx(x)[0][0]

            # if y:
            #     self._data[x] = r_result[y]
            # else:
            #     self._data[x] = r_result[x]

        return self

    def __getattr__(self, key):
        if not key.startswith("_"):
            try:
                return object.__getattribute__(self, "_data")[key]
            except KeyError:
                pass
        return getattr(self._data, key)

    def keys(self):
        return list(self._data.keys())

    def values(self):
        return list(self._data.values())

    def __len__(self):
        return self._data.__len__()

    def __str__(self):
        return str(self._data)

    def __contains__(self, key):
        return key in self._data

    def __getitem__(self, key):
        return self._data[key]

    def __delitem__(self, key):
        del self._data[key]

    def __setitem__(self, key, value):
        self._data[key] = value

    def __setattr__(self, key, value):
        if not key.startswith("_"):
            self._data[key] = value
        else:
            object.__setattr__(self, key, value)


class Summary(Result):

    """a collection of distributional parameters. Available properties
    are:

    mean, median, min, max, samplestd, sum, counts
    """

    fields = ("nval", "min", "max", "mean",
              "median", "stddev", "sum", "q1", "q3")

    def __init__(self, values=None,
                 format="%6.4f", mode="float",
                 allow_empty=True):

        Result.__init__(self)
        self._format = format
        self._mode = mode

        # note that this determintes the order of the fields at output
        self.counts, self.min, self.max, self.mean, self.median, self.samplestd, self.sum, self.q1, self.q3 = \
            (0, 0, 0, 0, 0, 0, 0, 0, 0)

        if values != None:

            values = [x for x in values if x != None]

            if len(values) == 0:
                if allow_empty:
                    return
                else:
                    raise ValueError("no data for statistics")

            # convert
            self._nerrors = 0
            if type(values[0]) not in (int, float):
                n = []
                for x in values:
                    try:
                        n.append(float(x))
                    except ValueError:
                        self._nerrors += 1
            else:
                n = values

            # use a non-sort algorithm?
            n.sort()
            if len(n):
                self.q1 = n[len(n) / 4]
                self.q3 = n[len(n) * 3 / 4]
            else:
                self.q1 = self.q3 = 0

            self.counts = len(n)
            self.min = min(n)
            self.max = max(n)
            self.mean = numpy.mean(n)
            self.median = numpy.median(n)
            self.samplestd = numpy.std(n)
            self.sum = reduce(lambda x, y: x + y, n)

    def getHeaders(self):
        """returns header of column separated values."""
        return self.fields

    def getHeader(self):
        """returns header of column separated values."""
        return "\t".join(self.getHeaders())

    def __str__(self):
        """return string representation of data."""

        if self._mode == "int":
            format_vals = "%i"
            format_median = "%.1f"
        else:
            format_vals = self._format
            format_median = self._format

        return "\t".join(("%i" % self.counts,
                          format_vals % self.min,
                          format_vals % self.max,
                          self._format % self.mean,
                          format_median % self.median,
                          self._format % self.samplestd,
                          format_vals % self.sum,
                          format_vals % self.q1,
                          format_vals % self.q3,
                          ))



# cython: embedsignature=True
# cython: profile=False

from libc.stdlib cimport qsort, calloc, malloc, realloc

cimport cython
from libc.stdint cimport UINT16_MAX

from SegmentList cimport Position, SegmentList

# trick to permit const void * in function definitions
cdef extern from *:
    ctypedef void * const_void_ptr "const void*"

@cython.profile(False)
cdef int cmpPosition(const_void_ptr s1, const_void_ptr s2) nogil: 
    return (<Position*>s1)[0] - (<Position*>s2)[0]


cdef class PositionList:
    '''list of segments.

    A list of (non-overlapping) segments.

    The intersection algorithms assume that segments
    are non-overlapping and sorted by start.

    Call normalize to merge overlapping segments.
    '''
    def __init__(self, 
                 int allocate=0,
                 SegmentList clone=None,
                 iter=None,
                 normalize=False,
                 SegmentList share=None,
                 unreduce=None):
        '''create empty list of segments.
        
        Creating is a lazy operation, no memory is allocated unless
        explitely given. A SegmentList might be initialized using
        the following options:

        If *allocate* is set, create an empty list, but allocate slots

        If *clone* is given, create a copy an existing list.

        If *iter* is given, initialize list from iterable.

        If *share* is set, the list will be a slave. It will share
        memory with a _master_, which has been set up by calling the 
        :meth:`share` method. Memory access for the slave is read-only.

        If *unreduce* is given, the list will be re-constituted
        from a pickled data representation (see __reduce__). Note that
        if a _master_ has been pickled, the reconstituted object will be
        a _slave_.

        If *normalize* is set, the list will be normalized.
        '''
        # initialize empty list
        self.positions = NULL
        self.npositions = 0
        self.allocated = 0

        # an empty list is normalized and thus sorted
        self.is_normalized = 1
        self.is_sorted = 1

    cdef _add(self, Position position):
        '''add a new position.

        The list will not be normalized automatically - call
        :meth:`PositionList.normalize`.
        '''

        if self.allocated == 0:
            self.positions = <Position*>malloc(self.chunk_size * sizeof(Position))
            if not self.positions:
                raise MemoryError("out of memory when allocation %i bytes" %
                                  sizeof( self.chunk_size * sizeof(Position)))
            self.allocated = self.chunk_size
        elif self.npositions == self.allocated:
            self.allocated *= 2
            self.positions = <Position*>realloc(self.positions,
                                              self.allocated * sizeof(Position))
            if not self.positions:
                raise MemoryError(
                    "out of memory when allocation %i bytes" %
                    sizeof(self.allocated * sizeof(Position)))

        self.positions[self.npositions] = position
        self.npositions += 1
        self.flag = 0

    def fromSegmentList(self, SegmentList segments, method="midpoint"):
        """fill from SegmentList.

        Valid methods are ``midpoint``, ``start`` and ``end``.

        Arguments
        ---------
        segments : SegmentList
             Segments to take from.
        method : string
             Method to convert segments to points.
        """
        self.clear()

        cdef int idx

        self.npositions = self.allocated = len(segments)
        self.positions = <Position*>calloc(self.npositions, sizeof(Position))
        idx = 0
        if method == "midpoint":
            for start, end in segments:
                if start == end:
                    continue
                self.points[idx] = start + (end - start) // 2
                idx += 1
        elif method == "start":
            for start, end in segments:
                if start == end:
                    continue
                self.points[idx] = start
                idx += 1
        elif method == "end":
            for start, end in segments:
                if start == end:
                    continue
                self.points[idx] = end
                idx += 1

        if segments.isNormalized():
            self.is_sorted = True
            self.is_normalized = True
    
        else:
            self.is_sorted = False
            self.is_normalized = False

        self.npositions = idx

    cpdef sort(self):
        """sort points."""
        if self.npositions == 0:
            return

        qsort(<void*>self.positions,
              self.npositions,
              sizeof(Position),
              &cmpPosition)

        self.is_sorted = True

    cpdef normalize(self):
        """normalized Point list.

        Overlapping points will be merged.
        """
        if self.npositions == 0:
            self.is_normalized = True
            return

        self.sort()

        cdef int idx = 0
        cdef int insertion_idx = 0
        cdef Position last_position
        cdef Position current_position
        for idx from 0 <= idx < self.npositions:
            current_position = self.positions[idx]
            if current_position != last_position:
                self.positions[insertion_idx] = current_position
                last_position = current_position
                insertion_index += 1
            idx += 1

        self.npositions = insertion_idx
        self._resize(self.npositions)
        self.is_normalized = True
        
    cdef _resize(self, int npositions):
        """resize position list to npositions."""
        # do not free all if there are no positions
        if npositions == 0:
            npositions = 1

        assert npositions >= self.npositions, "resizing will loose positions"
        
        if self.allocated == 0:
            self.positions = <Position*>malloc(npositions * sizeof(Position))
            if not self.positions:
                raise MemoryError("out of memory when allocation %i bytes" %
                                  sizeof(npositions * sizeof(Position)))
        else:
            self.positions = <Position*>realloc(self.positions,
                                               npositions * sizeof(Position))
            if not self.positions:
                raise MemoryError("out of memory when allocation %i bytes" %
                                  sizeof(self.npositions * sizeof(Position)))

        self.allocated = npositions

    cdef int _getInsertionPoint(self, Position other):
        """return insertion point.

        The insertion point denotes the element at which
        or after which `other` should be inserted to keep
        the sort order.

        This method does not check if the list is sorted. Peform
        this check before calling _getInsertionPoint.

        Returns
        -------
        position : int
           The function returns -1 if other is before the first element
           or the list is empty. The function returns npositions if other
           is after the last element.
        """

        cdef int idx
        if self.npositions == 0:
            return -1

        # avoid out of range searches
        if other.start >= self.positions[self.npositions-1].end:
            return self.npositions
        if other.end <= self.positions[0].start: 
            return -1

        idx = searchsorted(self.positions,
                           self.npositions,
                           sizeof(Position),
                           &other,
                           &cmpPosition)

        if idx == self.npositions:
            return idx-1
        elif self.positions[idx].start != other.start:
            return idx-1
        else:
            return idx

    cdef Position overlap(self, Position other):
        """return the number of nucleotides overlapping with other.

        Arguments
        ---------
        other : Position
            Position to compute overlap with

        Returns
        ------
        overlap : int
            Number of residues overlapping
        """
        assert self.is_sorted, "PointList is not sorted"

        cdef int idx
        idx = self._getInsertionPoint(other)

        # deal with border cases
        # permit partial overlap
        if idx == self.npositions:
            idx -= 1
        elif idx == -1:
            idx=0

        cdef Position count = 0

        while idx < self.npositions and self.positions[idx] == other:
            count += 1
            idx += 1
        return count
    



##########################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
##########################################################################
'''
Experiment.py - record keeping of experiments
=============================================

Module for record keeping of experiments. This module
is imported by most CGAT scripts. It provides convenience
methods for

   * argument parsing
   * record keeping (logging)
   * benchmarking

See :doc:`../scripts/script_template` on how to use this module.

API
---
'''

import re
import sys
import time
import inspect
import os
import optparse
import logging
import collections
import copy
import subprocess
import gzip
import uuid


class DefaultOptions:
    stdlog = sys.stdout
    stdout = sys.stdout
    stderr = sys.stderr

global_starting_time = time.time()
global_options = DefaultOptions()
global_args = None
global_id = uuid.uuid4()
global_benchmark = collections.defaultdict(int)


def openFile(filename, mode="r", create_dir=False):
    '''open file in *filename* with mode *mode*.

    If *create* is set, the directory containing filename
    will be created if it does not exist.

    gzip - compressed files are recognized by the
    suffix ``.gz`` and opened transparently.

    Note that there are differences in the file
    like objects returned, for example in the
    ability to seek.

    returns a file or file-like object.
    '''

    _, ext = os.path.splitext(filename)

    if create_dir:
        dirname = os.path.dirname(filename)
        if dirname and not os.path.exists(dirname):
            os.makedirs(dirname)

    if ext.lower() in (".gz", ".z"):
        return gzip.open(filename, mode)
    else:
        return open(filename, mode)


def getHeader():
    """return a header string with command line options and
    timestamp"""
    system, host, release, version, machine = os.uname()
    return "# output generated by %s\n# job started at %s on %s -- %s\n# pid: %i, system: %s %s %s %s" %\
           (" ".join(sys.argv),
            time.asctime(time.localtime(time.time())),
            host,
            global_id,
            os.getpid(),
            system, release, version, machine)


def getParams(options=None):
    """return a string containing script parameters.
    Parameters are all variables that start with "param_".
    """
    result = []
    if options:
        members = options.__dict__
        for k, v in sorted(members.items()):
            result.append("# %-40s: %s" % (k, str(v)))
    else:
        vars = inspect.currentframe().f_back.f_locals
        for var in [x for x in list(vars.keys()) if re.match("param_", x)]:
            result.append("# %-40s: %s" %
                          (var, str(vars[var])))

    if result:
        return "\n".join(result)
    else:
        return "# no parameters."


def getFooter():
    """return a header string with command line options and
    timestamp."""
    return "# job finished in %i seconds at %s -- %s -- %s" %\
           (time.time() - global_starting_time,
            time.asctime(time.localtime(time.time())),
            " ".join(["%5.2f" % x for x in os.times()[:4]]),
            global_id)


class OptionGroup(optparse.OptionGroup):
    pass


def callbackShortHelp(option, opt, value, parser):
    '''output short help (only command line options).'''
    # clear usage and description
    parser.set_description(None)
    parser.set_usage(None)
    # output help
    parser.print_help()
    # exit
    parser.exit()


class MultiLineFormatter(logging.Formatter):

    '''logfile formatter: add identation for multi-line entries.'''

    def format(self, record):
        s = logging.Formatter.format(self, record)
        if s.startswith("#"):
            prefix = "#"
        else:
            prefix = ""
        if record.message:
            header, footer = s.split(record.message)
            s = prefix + s.replace('\n', '\n%s' % prefix + ' ' * len(header))
        return s


def Start(parser=None,
          argv=sys.argv,
          quiet=False,
          no_parsing=False,
          add_csv_options=False,
          add_mysql_options=False,
          add_psql_options=False,
          add_pipe_options=True,
          add_cluster_options=False,
          add_output_options=False,
          return_parser=False):
    """set up an experiment.

    returns a tuple containing (options, args).

    The options class is extended with a logger module.
    """

    if not parser:
        parser = OptionParser(
            version="%prog version: $Id$")

    global global_options, global_args, global_starting_time

    # save default values given by user
    user_defaults = copy.copy(parser.defaults)

    global_starting_time = time.time()

    group = OptionGroup(parser, "Script timing options")

    group.add_option("--timeit", dest='timeit_file', type="string",
                     help="store timeing information in file [%default].")
    group.add_option("--timeit-name", dest='timeit_name', type="string",
                     help="name in timing file for this class of jobs "
                     "[%default].")
    group.add_option("--timeit-header", dest='timeit_header',
                     action="store_true",
                     help="add header for timing information [%default].")
    parser.add_option_group(group)

    group = OptionGroup(parser, "Common options")

    group.add_option("-v", "--verbose", dest="loglevel", type="int",
                     help="loglevel [%default]. The higher, the more output.")

    group.add_option("-?", dest="short_help", action="callback",
                     callback=callbackShortHelp,
                     help="output short help (command line options only.")

    parser.add_option_group(group)

    if quiet:
        parser.set_defaults(loglevel=0)
    else:
        parser.set_defaults(loglevel=1)

    parser.set_defaults(
        timeit_file=None,
        timeit_name='all',
        timeit_header=None,
    )

    if add_csv_options:
        parser.add_option("--csv-dialect", dest="csv_dialect", type="string",
                          help="csv dialect to use [%default].")

        parser.set_defaults(
            csv_dialect="excel-tab",
            csv_lineterminator="\n",
        )

    if add_psql_options:
        group = OptionGroup(parser, "postgres options")
        group.add_option("-C", "--connection", dest="psql_connection",
                         type="string",
                         help="psql connection string [%default].")
        group.add_option("-U", "--user", dest="user", type="string",
                         help="database user name [%default].")
        parser.set_defaults(psql_connection="fgu202:postgres")
        parser.set_defaults(user="")
        parser.add_option_group(group)

    if add_cluster_options:
        group = OptionGroup(parser, "cluster options")
        group.add_option("--no-cluster", "--local", dest="without_cluster",
                         action="store_true",
                         help="do no use cluster - run locally [%default].")
        group.add_option("--cluster-priority", dest="cluster_priority",
                         type="int",
                         help="set job priority on cluster [%default].")
        group.add_option("--cluster-queue", dest="cluster_queue",
                         type="string",
                         help="set cluster queue [%default].")
        group.add_option("--cluster-num-jobs", dest="cluster_num_jobs",
                         type="int",
                         help="number of jobs to submit to the queue execute "
                         "in parallel [%default].")
        group.add_option("--cluster-parallel",
                         dest="cluster_parallel_environment",
                         type="string",
                         help="name of the parallel environment to use "
                         "[%default].")
        group.add_option("--cluster-options", dest="cluster_options",
                         type="string",
                         help="additional options for cluster jobs, passed "
                         "on to queuing system [%default].")

        parser.set_defaults(without_cluster=False,
                            cluster_queue=None,
                            cluster_priority=None,
                            cluster_num_jobs=None,
                            cluster_parallel_environment=None,
                            cluster_options=None)
        parser.add_option_group(group)

    if add_output_options or add_pipe_options:
        group = OptionGroup(parser, "Input/output options")

        if add_output_options:
            group.add_option(
                "-P", "--output-filename-pattern",
                dest="output_filename_pattern", type="string",
                help="OUTPUT filename pattern for various methods "
                "[%default].")

            group.add_option("-F", "--force-output", dest="output_force",
                             action="store_true",
                             help="force over-writing of existing files.")

            parser.set_defaults(output_filename_pattern="%s",
                                output_force=False)

        if add_pipe_options:

            group.add_option("-I", "--stdin", dest="stdin", type="string",
                             help="file to read stdin from [default = stdin].",
                             metavar="FILE")
            group.add_option("-L", "--log", dest="stdlog", type="string",
                             help="file with logging information "
                             "[default = stdout].",
                             metavar="FILE")
            group.add_option("-E", "--error", dest="stderr", type="string",
                             help="file with error information "
                             "[default = stderr].",
                             metavar="FILE")
            group.add_option("-S", "--stdout", dest="stdout", type="string",
                             help="file where output is to go "
                             "[default = stdout].",
                             metavar="FILE")

            parser.set_defaults(stderr=sys.stderr)
            parser.set_defaults(stdout=sys.stdout)
            parser.set_defaults(stdlog=sys.stdout)
            parser.set_defaults(stdin=sys.stdin)

        parser.add_option_group(group)

    if add_mysql_options:
        group = OptionGroup(parser, "MYSQL connection options")
        group.add_option("-H", "--host", dest="host", type="string",
                         help="mysql host [%default].")
        group.add_option("-D", "--database", dest="database", type="string",
                         help="mysql database [%default].")
        group.add_option("-U", "--user", dest="user", type="string",
                         help="mysql username [%default].")
        group.add_option("-P", "--password", dest="password", type="string",
                         help="mysql password [%default].")
        group.add_option("-O", "--port", dest="port", type="int",
                         help="mysql port [%default].")

        parser.set_defaults(host="db",
                            port=3306,
                            user="",
                            password="",
                            database="")
        parser.add_option_group(group)

    # restore user defaults
    parser.defaults.update(user_defaults)

    if return_parser:
        return parser

    if not no_parsing:
        (global_options, global_args) = parser.parse_args(argv[1:])

    if add_pipe_options:
        if global_options.stdout != sys.stdout:
            global_options.stdout = openFile(global_options.stdout, "w")
        if global_options.stderr != sys.stderr:
            if global_options.stderr == "stderr":
                global_options.stderr = global_options.stderr
            else:
                global_options.stderr = openFile(global_options.stderr, "w")
        if global_options.stdlog != sys.stdout:
            global_options.stdlog = openFile(global_options.stdlog, "a")
        if global_options.stdin != sys.stdin:
            global_options.stdin = openFile(global_options.stdin, "r")
    else:
        global_options.stderr = sys.stderr
        global_options.stdout = sys.stdout
        global_options.stdlog = sys.stdout
        global_options.stdin = sys.stdin

    if global_options.loglevel >= 1:
        global_options.stdlog.write(getHeader() + "\n")
        global_options.stdlog.write(getParams(global_options) + "\n")
        global_options.stdlog.flush()

    # configure logging
    # map from 0-10 to logging scale
    # 0: quiet
    # 1: little verbositiy
    # >1: increased verbosity
    if global_options.loglevel == 0:
        lvl = logging.ERROR
    elif global_options.loglevel == 1:
        lvl = logging.INFO
    else:
        lvl = logging.DEBUG

    if global_options.stdout == global_options.stdlog:
        format = '# %(asctime)s %(levelname)s %(message)s'
    else:
        format = '%(asctime)s %(levelname)s %(message)s'

    logging.basicConfig(
        level=lvl,
        format=format,
        stream=global_options.stdlog)

    # set up multi-line logging
    # Note that .handlers is not part of the API, might change
    # Solution is to configure handlers explicitely.
    for handler in logging.getLogger().handlers:
        handler.setFormatter(MultiLineFormatter(format))

    return global_options, global_args


def Stop():
    """stop the experiment."""

    if global_options.loglevel >= 1 and global_benchmark:
        t = time.time() - global_starting_time
        global_options.stdlog.write(
            "######### Time spent in benchmarked functions ###################\n")
        global_options.stdlog.write("# function\tseconds\tpercent\n")
        for key, value in list(global_benchmark.items()):
            global_options.stdlog.write(
                "# %s\t%6i\t%5.2f%%\n" % (key, value, (100.0 * float(value) / t)))
        global_options.stdlog.write(
            "#################################################################\n")

    if global_options.loglevel >= 1:
        global_options.stdlog.write(getFooter() + "\n")

    # close files
    if global_options.stdout != sys.stdout:
        global_options.stdout.close()
    # do not close log, otherwise the following error occurs:
    # Error in sys.exitfunc:
    # Traceback (most recent call last):
    #   File "/net/cpp-group/server/lib/python2.6/atexit.py", line 24, in _run_exitfuncs
    #     func(*targs, **kargs)
    #   File "/net/cpp-group/server/lib/python2.6/logging/__init__.py", line 1472, in shutdown
    #     h.flush()
    #   File "/net/cpp-group/server/lib/python2.6/logging/__init__.py", line 740, in flush
    #     self.stream.flush()
    # ValueError: I/O operation on closed file
    # if global_options.stdlog != sys.stdout: global_options.stdlog.close()
    if global_options.stderr != sys.stderr:
        global_options.stderr.close()

    if global_options.timeit_file:

        outfile = open(global_options.timeit_file, "a")

        if global_options.timeit_header:
            outfile.write("\t".join(("name", "wall", "user", "sys", "cuser", "csys",
                                     "host", "system", "release", "machine",
                                     "start", "end", "path", "cmd")) + "\n")

        csystem, host, release, version, machine = list(map(str, os.uname()))
        uusr, usys, c_usr, c_sys = ["%5.2f" % x for x in os.times()[:4]]
        t_end = time.time()
        c_wall = "%5.2f" % (t_end - global_starting_time)

        if sys.argv[0] == "run.py":
            cmd = global_args[0]
            if len(global_args) > 1:
                cmd += " '" + "' '".join(global_args[1:]) + "'"
        else:
            cmd = sys.argv[0]

        result = "\t".join((global_options.timeit_name,
                            c_wall, uusr, usys, c_usr, c_sys,
                            host, csystem, release, machine,
                            time.asctime(time.localtime(global_starting_time)),
                            time.asctime(time.localtime(t_end)),
                            os.path.abspath(os.getcwd()),
                            cmd)) + "\n"

        outfile.write(result)
        outfile.close()


def benchmark(func):
    """decorator collecting wall clock time spent in decorated method."""

    def wrapper(*arg):
        t1 = time.time()
        res = func(*arg)
        t2 = time.time()
        key = "%s:%i" % (func.__name__, func.__code__.co_firstlineno)
        global_benchmark[key] += t2 - t1
        global_options.stdlog.write(
            '## benchmark: %s completed in %6.4f s\n' % (key, (t2 - t1)))
        global_options.stdlog.flush()
        return res
    return wrapper

# there are differences whether you cache a function or
# an objects method


def cachedmethod(function):
    '''decorator for caching a method.'''
    return Memoize(function)


class Memoize(object):

    def __init__(self, fn):
        self.cache = {}
        self.fn = fn

    def __get__(self, instance, cls=None):
        self.instance = instance
        return self

    def __call__(self, *args):
        if args in self.cache:
            return self.cache[args]
        else:
            object = self.cache[args] = self.fn(self.instance, *args)
            return object


def log(loglevel, message):
    """log message at loglevel."""
    logging.log(loglevel, message)


def info(message):
    '''log information message, see the :mod:`logging` module'''
    logging.info(message)


def warning(message):
    '''log warning message, see the :mod:`logging` module'''
    logging.warning(message)


def warn(message):
    '''log warning message, see the :mod:`logging` module'''
    logging.warning(message)


def debug(message):
    '''log debugging message, see the :mod:`logging` module'''
    logging.debug(message)


def error(message):
    '''log error message, see the :mod:`logging` module'''
    logging.error(message)


def critical(message):
    '''log critical message, see the :mod:`logging` module'''
    logging.critical(message)


def getOutputFile(section):
    '''return filename to write to.'''
    return re.sub("%s", section, global_options.output_filename_pattern)


def openOutputFile(section, mode="w"):
    """open file for writing substituting section in the
    output_pattern (if defined).

    If the filename ends with ".gz", the output is opened
    as a gzip'ed file.
    """

    fn = getOutputFile(section)
    try:
        if fn == "-":
            return global_options.stdout
        else:
            if not global_options.output_force and os.path.exists(fn):
                raise OSError(
                    "file %s already exists, use --force to overwrite existing files." % fn)
            if fn.endswith(".gz"):
                return gzip.open(fn, mode)
            else:
                return open(fn, mode)
    except AttributeError:
        return global_options.stdout


class Counter(object):

    '''a counter class.

    The counter acts both as a dictionary and
    a object permitting attribute access.

    Counts are automatically initialized to 0.

    Instantiate and use like this::

       c = Counter()
       c.input += 1
       c.output += 2
       c["skipped"] += 1

       print str(c)
    '''

    __slots__ = ["_counts"]

    def __init__(self):
        """Store data returned by function."""
        object.__setattr__(self, "_counts", collections.defaultdict(int))

    def __setitem__(self, key, value):
        self._counts[key] = value

    def __getitem__(self, key):
        return self._counts[key]

    def __getattr__(self, name):
        return self._counts[name]

    def __setattr__(self, name, value):
        self._counts[name] = value

    def __str__(self):
        return ", ".join("%s=%i" % x for x in self._counts.items())

    def __iadd__(self, other):
        try:
            for key, val in other.items():
                self._counts[key] += val
        except:
            raise TypeError("unknown type")
        return self

    def iteritems(self):
        return iter(self._counts.items())


def run(cmd):
    '''executed a command line cmd.

    raises OSError if process failed or was terminated.
    '''

    retcode = subprocess.call(cmd, shell=True)
    if retcode < 0:
        raise OSError("process was terminated by signal %i" % -retcode)
    return retcode



# cython: embedsignature=True
# cython: profile=False

cimport cython

from cpython.bytes cimport PyBytes_AsString, PyBytes_FromStringAndSize
from cpython cimport PyBytes_Check, PyUnicode_Check
from libc.stdlib cimport qsort, calloc, malloc, realloc, free
from libc.string cimport memcpy
from libc.errno cimport errno
from posix.types cimport off_t
from posix.mman cimport mmap, munmap, shm_open, shm_unlink
from posix.mman cimport MAP_SHARED, PROT_READ, PROT_WRITE
from posix.stat cimport S_IRUSR, S_IWUSR
from posix.fcntl cimport O_CREAT, O_RDWR, O_RDONLY

from SegmentList cimport Position, Segment, SegmentList

cdef bytes force_bytes(object s, encoding="ascii"):
    """convert string or unicode object to bytes, assuming
    ascii encoding.
    """
    if s is None:
        return None
    elif PyBytes_Check(s):
        return s
    elif PyUnicode_Check(s):
        return s.encode(encoding)
    else:
        raise TypeError("Argument must be string, bytes or unicode.")


# trick to permit const void * in function definitions
cdef extern from *:
    ctypedef void * const_void_ptr "const void*"

@cython.profile(False)
cdef int cmpPosition(const_void_ptr s1, const_void_ptr s2) nogil: 
    return (<Position*>s1)[0] - (<Position*>s2)[0]


cdef class PositionList:
    '''list of segments.

    A list of (non-overlapping) segments.

    The intersection algorithms assume that segments
    are non-overlapping and sorted by start.

    Call normalize to merge overlapping segments.
    '''
    def __init__(self, 
                 int allocate=0,
                 PositionList clone=None,
                 iter=None,
                 sort=False,
                 normalize=False,
                 PositionList share=None,
                 unreduce=None):
        '''create empty list of segments.
        
        Creating is a lazy operation, no memory is allocated unless
        explitely given. A SegmentList might be initialized using
        the following options:

        If *allocate* is set, create an empty list, but allocate slots

        If *clone* is given, create a copy an existing list.

        If *iter* is given, initialize list from iterable.

        If *share* is set, the list will be a slave. It will share
        memory with a _master_, which has been set up by calling the 
        :meth:`share` method. Memory access for the slave is read-only.

        If *unreduce* is given, the list will be re-constituted
        from a pickled data representation (see __reduce__). Note that
        if a _master_ has been pickled, the reconstituted object will be
        a _slave_.

        If *normalize* is set, the list will be normalized.
        '''
        # initialize empty list
        self.positions = NULL
        self.npositions = 0
        self.allocated = 0
        self.chunk_size = 10000

        # an empty list is normalized and thus sorted
        self.is_normalized = 1
        self.is_sorted = 1
        self.is_shared = False
        self.is_slave = False
        self.shared_fd = -1
        self.key = None
        cdef int idx
        cdef Position pos
        cdef void * retval

        # initialize list from shared memory
        # allocated is 0
        if share != None:
            self.npositions = share.npositions
            if self.npositions > 0:
                assert share.shared_fd >= 0, "sharing from a non-shared SegmentList"

                self.allocated = 0
                self.is_sorted = share.is_sorted
                self.is_normalized = share.is_normalized
                self.is_shared = True
                self.is_slave = True
                self.chunk_size = share.chunk_size
                retval = <Position*>mmap(NULL, 
                                         self.npositions * sizeof(Segment),
                                         PROT_READ, 
                                         MAP_SHARED, 
                                         share.shared_fd, 
                                         0)

                error_number = errno

                if retval == <void*>-1:
                    raise ValueError(
                        "could not read list from shared segment - error=%i" %
                        error_number )
                self.positions = <Position*>retval

        # create from pickled representation
        # if a shared object is pickled, it will be re-constituted 
        # as a slave list.
        elif unreduce:
            self.npositions, self.allocated, self.is_sorted, \
                self.is_normalized, \
                self.chunk_size, self.key = unreduce[:6]
            if type(unreduce[6]) == int:
                # shared memory
                shared_fd = unreduce[6]
                
                retval = mmap(NULL, 
                              self.npositions * sizeof(Position),
                              PROT_READ, 
                              MAP_SHARED, 
                              shared_fd, 
                              0)

                error_number = errno

                if retval == <void*>-1:
                    raise ValueError(
                        "could not unpickle as slave - error=%i" % error_number)

                self.positions = <Position *>retval

                # mark memory as slave - nothing allocated and no shared_fd
                self.allocated = 0
                self.shared_fd = -1
                self.is_shared = True
                self.is_slave = True
            else:
                p = PyBytes_AsString(unreduce[6])
                self.positions = <Position*>malloc(self.npositions * sizeof(Position))
                memcpy(self.positions, p, cython.sizeof(Position) * self.npositions)

        # clone from another list
        elif clone != None:
            self.npositions = self.allocated = clone.npositions
            self.positions = <Position*>calloc(clone.npositions, sizeof(Position))
            memcpy(self.positions,
                   clone.positions,
                   clone.npositions * sizeof(Position))
            self.is_sorted = clone.is_sorted
            self.is_normalized = clone.is_normalized
            
        # create from an iterable
        elif iter:
            a = tuple(iter)
            self.npositions = self.allocated = len(a)
            self.positions = <Position*>calloc(self.npositions, sizeof(Position))
            idx = 0
            for pos in a:
                self.positions[idx] = pos
                idx += 1
            self.is_normalized = False
            self.is_sorted = False

        # allocated memory only, list remains empty
        elif allocate:
            self.allocated = allocate
            self.positions = <Position*>calloc(allocate, sizeof(Position))

        if sort:
            self.sort()

        if normalize:
            self.normalize()

    def __reduce__(self):
        '''pickling function - returns class contents as a tuple.'''

        cdef bytes data

        if self.shared_fd >= 0:
            return (buildPositionList, (self.npositions, 
                                        self.allocated, 
                                        self.is_sorted,
                                        self.is_normalized,
                                        self.chunk_size, 
                                        self.key,
                                        self.shared_fd))

        else:
            data = PyBytes_FromStringAndSize(
                <char*>self.positions, \
                self.npositions * cython.sizeof(Position) * 2)
        
            return (buildPositionList, (self.npositions, 
                                        self.allocated,
                                        self.is_sorted,
                                        self.is_normalized,
                                        self.chunk_size, 
                                        self.key,
                                        data))
    cpdef Position max(self):
        '''return maximum coordinate.'''
        assert self.is_sorted, "PositionList : max from unsorted list"
        if self.npositions == 0:
            return 0
        return self.positions[self.npositions - 1]

    cpdef Position min(self):
        '''return minimum coordinate.'''
        assert self.is_sorted, "PositionList : min from unsorted list"
        if self.npositions == 0:
            return 0
        return self.positions[0]

    cpdef Position sum(self):
        '''return the sum (number) of positions.'''
        return self.npositions

    cpdef clear(self):
        '''removes all positions from list.

        Note that this method does not shrink the allocated memory.
        '''
        self.npositions = 0
        self.is_sorted = True
        self.is_normalized = True

    def __len__(self):
        return self.npositions

    cdef _add(self, Position position):
        '''add a new position.

        The list will not be normalized automatically - call
        :meth:`PositionList.normalize`.
        '''

        if self.allocated == 0:
            self.positions = <Position*>malloc(self.chunk_size * sizeof(Position))
            if not self.positions:
                raise MemoryError("out of memory when allocation %i bytes" %
                                  sizeof( self.chunk_size * sizeof(Position)))
            self.allocated = self.chunk_size
        elif self.npositions == self.allocated:
            self.allocated *= 2
            self.positions = <Position*>realloc(self.positions,
                                                self.allocated * sizeof(Position))
            if not self.positions:
                raise MemoryError(
                    "out of memory when allocation %i bytes" %
                    sizeof(self.allocated * sizeof(Position)))

        self.positions[self.npositions] = position
        self.npositions += 1
        self.is_sorted = False
        self.is_normalized = False

    cpdef add(self, Position pos):
        '''add a new position.

        The list will not be normalized automatically - call
        :meth:`PositionList.normalize`.
        '''
        self._add(pos)

    def fromSegmentList(self, SegmentList segments, method="midpoint"):
        """fill from SegmentList.

        Valid methods are ``midpoint``, ``start`` and ``end``.

        Arguments
        ---------
        segments : SegmentList
             Segments to take from.
        method : string
             Method to convert segments to points.
        """
        self.clear()

        cdef int idx

        self.npositions = self.allocated = len(segments)
        self.positions = <Position*>calloc(self.npositions, sizeof(Position))
        idx = 0
        if method == "midpoint":
            for start, end in segments:
                if start == end:
                    continue
                self.positions[idx] = start + (end - start) // 2
                idx += 1
        elif method == "start":
            for start, end in segments:
                if start == end:
                    continue
                self.positions[idx] = start
                idx += 1
        elif method == "end":
            for start, end in segments:
                if start == end:
                    continue
                self.positions[idx] = end
                idx += 1
        else:
            raise ValueError("unknow method '%s'" % method)

        if segments.isNormalized:
            self.is_sorted = True
            self.is_normalized = True
        else:
            self.is_sorted = False
            self.is_normalized = False

        self.npositions = idx

    cpdef sort(self):
        """sort points."""
        if self.npositions == 0:
            self.is_sorted = True
            return

        qsort(<void*>self.positions,
              self.npositions,
              sizeof(Position),
              &cmpPosition)

        self.is_sorted = True

    cpdef normalize(self):
        """normalized position list.

        Overlapping positions will be merged.
        """
        if self.npositions == 0:
            self.is_normalized = True
            return

        self.sort()

        cdef int idx = 0
        cdef int insertion_idx = 0
        cdef Position last_position = -1
        cdef Position current_position
        for idx from 0 <= idx < self.npositions:
            current_position = self.positions[idx]
            if current_position != last_position:
                self.positions[insertion_idx] = current_position
                last_position = current_position
                insertion_idx += 1

        self.npositions = insertion_idx
        self._resize(self.npositions)
        self.is_normalized = True
        
    cdef _resize(self, int npositions):
        """resize position list to npositions."""
        # do not free all if there are no positions
        if npositions == 0:
            npositions = 1

        assert npositions >= self.npositions, "resizing will loose positions"
        
        if self.allocated == 0:
            self.positions = <Position*>malloc(npositions * sizeof(Position))
            if not self.positions:
                raise MemoryError("out of memory when allocation %i bytes" %
                                  sizeof(npositions * sizeof(Position)))
        else:
            self.positions = <Position*>realloc(self.positions,
                                               npositions * sizeof(Position))
            if not self.positions:
                raise MemoryError("out of memory when allocation %i bytes" %
                                  sizeof(self.npositions * sizeof(Position)))

        self.allocated = npositions

    cdef int _getInsertionPoint(self, Position other):
        """return insertion point.

        The insertion point denotes the element at which
        or after which `other` should be inserted to keep
        the sort order.

        This method does not check if the list is sorted. Peform
        this check before calling _getInsertionPoint.

        Returns
        -------
        position : int
           The function returns -1 if other is before the first element
           or the list is empty. The function returns npositions if other
           is after the last element.
        """

        cdef int idx
        if self.npositions == 0:
            return -1

        # avoid out of range searches
        if other < self.positions[0]:
            return -1
        if other >= self.positions[self.npositions-1]:
            return self.npositions

        return searchsorted(self.positions,
                            self.npositions,
                            sizeof(Position),
                            &other,
                            &cmpPosition)

    cdef Position overlap(self, Segment other):
        """return the number of nucleotides overlapping with other.

        Arguments
        ---------
        other : Segment
            Segment to compute overlap with

        Returns
        ------
        overlap : int
            Number of residues overlapping
        """
        assert self.is_sorted, "PointList is not sorted"

        cdef Position count = 0
        cdef int idx = self._getInsertionPoint(other.start)
        # if beyond last element, check if overlap with
        # previous elements
        if idx == self.npositions:
            idx -= 1
            while idx > 0 and \
                  self.positions[idx] >= other.start and \
                  self.positions[idx] < other.end:
                idx -= 1
                count += 1
            return count

        if idx < 0:
            idx = 0

        while idx < self.npositions and \
              self.positions[idx] >= other.start and \
              self.positions[idx] < other.end:
            count += 1
            idx += 1
        return count
    
    cpdef Position overlapWithRange(self, Position start, Position end):
        """return the number of nucleotides overlapping with a range.

        Arguments
        ---------
        start : int
            Begin of range.
        end : int
            End of range.

        Returns
        ------
        overlap : int
            Number of residues overlapping
        """

        cdef Segment s
        s = Segment(start, end)
        return self.overlap(s)

    cpdef Position intersectionWithSegments(
        self,
        SegmentList other,
        mode="base"):
        """return number of positions overlapping with *other*.

        `mode` is there for compatibility with SegmentList.


        Arguments
        ---------
        other : SegmentList
             SegmentList to overlap with
        mode : string
             Unused.

        Returns
        -------
        overlap : int
            Number of positions
        """

        assert self.is_sorted, "Intersection from unsorted positions"
        assert other.isNormalized, "Intersection with non-normalized segments"

        cdef int this_idx, other_idx, last_this_idx, last_other_idx
        this_idx = other_idx = 0
        last_this_idx = last_other_idx = -1
        cdef Position this_position = -1
        cdef Segment other_segment = Segment(0,0)
        cdef Position noverlap = 0

        while this_idx < self.npositions and other_idx < other.nsegments:
            if last_this_idx != this_idx:
                this_position = self.positions[this_idx]
                last_this_idx = this_idx
            if last_other_idx != other_idx:
                other_segment = other.segments[other_idx]
                last_other_idx = other_idx

            # skip segments in this not overlapping other
            if this_position < other_segment.start:
                this_idx += 1
            # skip segments in other not overlapping this
            elif other_segment.end <= this_position:
                other_idx += 1
            else:
                noverlap += 1
                this_idx += 1

        return noverlap

    cpdef void intersect(self, SegmentList other):
        '''intersect this position list with a SegmentList.

        This method removes all Positions that are not contained
        in Segments in `other`.
        '''

        assert self.is_sorted, "Intersection from unsorted positions"
        assert other.isNormalized, "Intersection with non-normalized segments"

        if self.npositions == 0:
            return

        if len(other) == 0:
            self.clear()
            return

        cdef int this_idx, other_idx, last_this_idx, last_other_idx, working_idx
        this_idx = other_idx = working_idx = 0
        last_this_idx = last_other_idx = -1
        cdef Position this_position = -1
        cdef Segment other_segment = Segment(0,0)
        cdef Position noverlap = 0

        while this_idx < self.npositions and other_idx < other.nsegments:
            if last_this_idx != this_idx:
                this_position = self.positions[this_idx]
                last_this_idx = this_idx
            if last_other_idx != other_idx:
                other_segment = other.segments[other_idx]
                last_other_idx = other_idx

            # skip segments in this not overlapping other
            if this_position < other_segment.start:
                this_idx += 1
            # skip segments in other not overlapping this
            elif other_segment.end <= this_position:
                other_idx += 1
            else:
                self.positions[working_idx] = this_position
                working_idx += 1
                this_idx += 1

        self.npositions = working_idx

    def __str__(self):
        return str(self.asList())

    def asList(self):

        cdef int idx
        result = []
        for idx from 0 <= idx < self.npositions:
            result.append(self.positions[idx])
        return result

    cpdef PositionList clone(self):
        """return a copy of self."""
        clone = PositionList(allocate=self.npositions)
        memcpy(clone.positions,
               self.positions,
               self.npositions * sizeof(Position))
        clone.npositions = self.npositions
        clone.is_sorted = self.is_sorted
        clone.is_normalized = self.is_normalized
        return clone

    def __dealloc__(self):
        cdef int fd

        if self.positions != NULL:
            # unshared memory
            if self.is_shared:
                if self.is_slave:
                    pass
                else:
                    # free shared memory as master
                    munmap(self.positions, 
                           self.npositions * sizeof(Position))
            else:
                # free private copy
                free(self.positions)

        if self.shared_fd != -1:
            fd = shm_unlink(self.key)
            if fd == -1:
                raise OSError("could not unlink shared memory")

    cdef void fromMMAP(self):
        '''retrieve data from mmapped location to private copy.
        '''
        # copy data back to unshared memory
        cdef Position * s
        cdef off_t nbytes = sizeof(Position) * self.npositions
    
        if self.npositions == 0:
            return

        s = <Position *>malloc(nbytes)
        if s == NULL:
            raise ValueError( "could not allocate memory when unsharing" )
        
        memcpy(s, self.positions, nbytes)
        
        self.positions = s
        self.allocated = self.npositions
        self.is_shared = False
        self.is_slave = False

    cdef off_t toMMAP(self, 
                      void * mmap, 
                      int shared_fd, 
                      off_t offset):
        '''move contents of list to shared memory 
        at *mmap* with *offset*.
        '''
        if self.shared_fd != -1: 
            raise ValueError("can not move shared PositionList to mmap")

        if self.npositions == 0:
            # do not move empty position lists
            return offset
        
        cdef off_t nbytes = sizeof(Position) * self.npositions
        cdef Position * p = <Position *>mmap + offset

        # copy data
        memcpy(p, self.positions, nbytes)

        # free allocated private memory
        self.allocated = 0
        free(self.positions)
        self.positions = p

        self.is_shared = True
        self.is_slave = True

        return offset + self.npositions

def buildPositionList(*args):
    '''pickling helper function.
    
    Return a re-constructed SegmentList object.
    '''
    return PositionList(unreduce=args)




"""test gat implementation."""

import unittest
import random
import os
import numpy
import pickle

from gat.PositionList import PositionList
from gat.SegmentList import SegmentList


class GatTest(unittest.TestCase):

    def shortDescription(self):
        return None


class TestPositionList(GatTest):
    '''test segment list implementation.'''

    def testCreateAndClear(self):
        s = PositionList()
        self.assertEqual(0, len(s))
        s.add(100)
        self.assertEqual(1, len(s))
        s.clear()
        self.assertEqual(0, len(s))

    def testNormalize(self):
        '''non-overlapping segments.'''

        ss = list(range(0, 1000, 100))
        random.shuffle(ss)
        s = PositionList()
        for pos in ss:
            s.add(pos)
        self.assertEqual(len(s), 10)
        s.normalize()
        self.assertEqual(len(s), 10)
        self.assertEqual(s.sum(), 10)
        for pos in ss:
            s.add(pos)
        self.assertEqual(len(s), 20)
        s.normalize()
        self.assertEqual(len(s), 10)

    def testFromSegments(self):
        l = [(489, 589),
             (1966, 2066),
             (2786, 2886),
             (3889, 3972),
             (3998, 4098),
             (6441, 6541),
             (6937, 7054),
             (7392, 7492),
             (8154, 8254),
             (9046, 9146)]
        ss = SegmentList(iter=l)
        pp = PositionList()
        pp.fromSegmentList(ss)
        self.assertEqual(len(ss), len(pp))

    def testOverlapWithRange(self):

        # single point per position
        pp = PositionList(iter=list(range(0, 1000, 100)), sort=True)

        for x in range(0, 1050, 10):
            if x % 100 == 0:
                if x == 1000:
                    self.assertEqual(0, pp.overlapWithRange(x, x+10))
                else:
                    self.assertEqual(1, pp.overlapWithRange(x, x+10))
            else:
                self.assertEqual(0, pp.overlapWithRange(x, x+10))

        # two points per positions
        pp = PositionList(iter=list(range(0, 1000, 100)) + list(range(0, 1000, 100)),
                          sort=True)

        for x in range(0, 1050, 10):
            if x % 100 == 0:
                if x == 1000:
                    self.assertEqual(0, pp.overlapWithRange(x, x+10))
                else:
                    self.assertEqual(2, pp.overlapWithRange(x, x+10))
            else:
                self.assertEqual(0, pp.overlapWithRange(x, x+10))

    def testOverlapWithSegments(self):

        # single point per position
        pp = PositionList(iter=list(range(0, 1000, 100)), sort=True)

        for o in range(0, 200, 10):
            ss = SegmentList(
                iter=[(x, x + 1) for x in range(0 + o, 1000 + o, 100)],
                normalize=True)
            if o % 100 == 0:
                if o == 100:
                    self.assertEqual(9,
                                     pp.intersectionWithSegments(ss))
                else:
                    self.assertEqual(10,
                                     pp.intersectionWithSegments(ss))
            else:
                self.assertEqual(0,
                                 pp.intersectionWithSegments(ss))

    def testIntersect(self):

        for o in range(0, 200, 10):
            # single point per position
            pp = PositionList(iter=list(range(0, 1000, 100)), sort=True)

            ss = SegmentList(
                iter=[(x, x + 1) for x in range(0 + o, 1000 + o, 100)],
                normalize=True)
            pp.intersect(ss)
            if o % 100 == 0:
                if o == 100:
                    self.assertEqual(9, len(pp))
                else:
                    self.assertEqual(10, len(pp))
            else:
                self.assertEqual(0, len(pp))

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
##########################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: script_template.py 2871 2010-03-03 10:20:44Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
##########################################################################
'''
gat-plot - plot results from a gat analysis
===========================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script takes the results of a ``gat-run.py` or ``gat-compare.py``
and plots the results.

This script requires matplotlib.

Usage
-----

Example::

   python gat-plot.py --input-filename-results=gat.results.tsv.gz
   python gat-plot.py --input-filename-counts=gat.counts.tsv.gz
 
Type::

   python gatplot.py --help

for command line help.

Documentation
-------------

Code
----

'''

import os
import sys
import re
import optparse
import collections
import types
import glob
import time
import numpy

import gat
import gat.Experiment as E
import gat.IOTools as IOTools
import gat.IO as IO

try:
    import matplotlib.pyplot as plt
    HASPLOT = True
except (ImportError, RuntimeError):
    HASPLOT = False


class DummyAnnotatorResult:

    format_observed = "%i"
    format_expected = "%6.4f"
    format_fold = "%6.4f"
    format_pvalue = "%6.4e"

    def __init__(self):
        pass

    @classmethod
    def _fromLine(cls, line):
        x = cls()
        data = line[:-1].split("\t")
        x.track, x.annotation = data[:2]
        x.observed, x.expected, x.lower95, x.upper95, x.stddev, x.fold, x.pvalue, x.qvalue = \
            map(float, data[2:])
        return x

    def __str__(self):
        return "\t".join((self.track,
                          self.annotation,
                          self.format_observed % self.observed,
                          self.format_expected % self.expected,
                          self.format_expected % self.lower95,
                          self.format_expected % self.upper95,
                          self.format_expected % self.stddev,
                          self.format_fold % self.fold,
                          self.format_pvalue % self.pvalue,
                          self.format_pvalue % self.qvalue))


def buildPlotFilename(options, key):
    filename = re.sub("%s", key, options.output_plots_pattern)
    filename = re.sub("[^a-zA-Z0-9-_./]", "_", filename)
    dirname = os.path.dirname(filename)
    if dirname and not os.path.exists(dirname):
        os.makedirs(dirname)
    return filename


def plotBarplots(annotator_results, options):
    '''output a series of bar-plots.

    Output for each track.

    Significant results are opaque, while
    non-significant results are transparent.'''

    for track in annotator_results:
        plt.figure()
        r = annotator_results[track]
        keys, values = zip(*r.items())
        pos = range(len(r))
        bars = plt.barh(pos, [x.fold for x in values])
        for b, v in zip(bars, values):
            if v.qvalue > 0.05:
                b.set_alpha(0.10)

        filename = buildPlotFilename(options, "bars-%s" % track)
        plt.yticks(pos, keys)
        plt.axvline(x=1, color="r")
        plt.savefig(filename)


def plotBarplot(annotator_results, options):
    '''output a single bar-plots.

    Output for each track.

    Significant results are opaque, while
    non-significant results are transparent.'''

    ntracks = len(annotator_results)
    height = 1.0 / float(ntracks)

    plt.figure()

    for trackid, track in enumerate(annotator_results):

        r = annotator_results[track]
        rr = r.items()
        rr.sort()
        keys, values = zip(*rr)
        pos = numpy.arange(0, len(r), 1) + trackid * height
        bars = plt.barh(pos,
                        [x.fold for x in values],
                        height=height,
                        label=track,
                        xerr=[x.stddev / x.expected for x in values],
                        color="bryg"[trackid % 4])
        for b, v in zip(bars, values):
            if v.pvalue > 0.05:
                b.set_alpha(0.10)

    pos = range(len(r))

    plt.yticks(pos, keys)
    plt.axvline(x=1, color="r")
    filename = buildPlotFilename(options, "bars-all")
    plt.legend()
    plt.savefig(filename)


def main(argv=None):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv:
        argv = sys.argv

    # setup command line parser
    parser = optparse.OptionParser(version="%prog version: $Id: script_template.py 2871 2010-03-03 10:20:44Z andreas $",
                                   usage=globals()["__doc__"])

    parser.add_option("-l", "--sample-file", dest="sample_files", type="string", action="append",
                      help="filename with sample files. Start processing from samples [default=%default].")

    parser.add_option("-o", "--order", dest="output_order", type="choice",
                      choices=(
                          "track", "annotation", "fold", "pvalue", "qvalue"),
                      help="order results in output by fold, track, etc. [default=%default].")

    parser.add_option("-p", "--pvalue-method", dest="pvalue_method", type="choice",
                      choices=("empirical", "norm", ),
                      help="type of pvalue reported [default=%default].")

    parser.add_option("--results-file", dest="input_filename_results", type="string",
                      help="start processing from results - no segments required [default=%default].")

    parser.add_option("--output-plots-pattern", dest="output_plots_pattern", type="string",
                      help="output pattern for plots [default=%default]")

    parser.add_option("--output-samples-pattern", dest="output_samples_pattern", type="string",
                      help="output pattern for samples. Samples are stored in bed format, one for "
                      " each segment [default=%default]")

    parser.add_option("--plots", dest="plots", type="choice",
                      choices=("all",
                               "bars-per-track",
                               "bars", ),
                      help="plots to be created [default=%default].")

    parser.set_defaults(
        sample_files=[],
        num_samples=1000,
        output_stats=[],
        output_filename_counts=None,
        output_order="fold",
        input_filename_results=None,
        pvalue_method="empirical",
        output_plots_pattern=None,
        plots=[],
    )

    # add common options (-h/--help, ...) and parse command line
    (options, args) = E.Start(parser, argv=argv, add_output_options=True)

    annotator_results = IO.readAnnotatorResults(options.input_filename_results)

    if "speparate-bars" in options.plots:
        plotBarplots(annotator_results, options)
    if "bars" in options.plots:
        plotBarplot(annotator_results, options)

    # write footer and output benchmark information.
    E.Stop()

if __name__ == "__main__":
    sys.exit(main(sys.argv))
